import { completeFromList } from '@codemirror/autocomplete';
import { StreamLanguage, LanguageSupport } from '@codemirror/language';
import { tags } from '@lezer/highlight';

const FireStoreLanguage = StreamLanguage.define({
    name: "firestore",
    startState: (indentUnit) => {
        return {};
    },
    token: (stream, state = {}) => {
        if (stream.match("db")) {
            state.db = true;
            return "namespace";
        }
        if (stream.match("collection")) {
            if (state.db) {
                state.collection = true;
                return "function";
            }
        }
        if (stream.match("where")) {
            if (state.collection) {
                state.where = true;
                return "function";
            }
        }
        if (stream.match("limit")) {
            if (state.where) {
                state.limit = true;
                return "function";
            }
        }
        if (stream.match("get")) {
            if (state.limit) {
                state.limit = false;
                return "function";
            }
        }
        if (stream.match(/"(?:[^\\"]|\\.)*"/)) {
            if (state.collection) {
                return "string";
            }
            if (state.where) {
                state.where = false;
                state.whereValue = true;
                return "string";
            }
            if (state.whereValue) {
                state.whereValue = false;
                return "string";
            }
            if (stream.match("==")) {
                if (state.whereValue) {
                    state.whereValue = false;
                    state.whereOperator = true;
                    return "operator";
                }
            }
            if (stream.match(/[0-9]+/)) {
                if (state.limit) {
                    return "number";
                }
            }
        }
        stream.next();
        return null;
    },
    blankLine: (state, indentUnit) => { },
    copyState: (state) => { },
    indent: (state, textAfter, context) => {
        return 1;
    },
    languageData: {
        commentTokens: { line: ";" },
    },
    tokenTable: {
        db: tags.namespace,
        dot: tags.punctuation,
        collection: tags.function(tags.variableName),
        where: tags.function(tags.variableName),
        limit: tags.function(tags.variableName),
        get: tags.function(tags.atom),
        lParen: tags.punctuation,
        rParen: tags.punctuation,
        string: tags.string,
    },
});
const FireStoreCompletion = FireStoreLanguage.data.of({
    autocomplete: completeFromList([
        { label: "db", type: "namespace" },
        { label: "collection", type: "function" },
        { label: "where", type: "function" },
        { label: "limit", type: "function" },
        { label: "get", type: "function" },
    ]),
});
function firestore() {
    return new LanguageSupport(FireStoreLanguage, [FireStoreCompletion]);
}

export { FireStoreCompletion, FireStoreLanguage, firestore };
